/*
 Copyright (C) JAEHYUK CHO
 All rights reserved.
 Code by JaeHyuk Cho <mailto:minzkn@minzkn.com>
*/

#if !defined(__def_mzapi_source_main_c__)
#define __def_mzapi_source_main_c__ "main.c"

#include <asm/param.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "mzps.h"

static void (mzps_title)(void)
{
 (void)fprintf(stdout,
  "  %3s %5s %5s %-8s %5s %7s %8s %8s %6s %-15s\n", 
  "Idx", "PPID", "PID", "UID", "State", "Threads", "VmSize", "RSS", "CPU", "Name");
}

int (main)(int s_argc, char **s_argv)
{
 struct ts_mzapi_process *s_process;
 struct ts_mzapi_process *s_prev_process;
 struct ts_mzapi_process *s_trace;
 struct ts_mzapi_process *s_prev_trace;
 int s_index, s_cpu_percent;
 pid_t s_pid;
 s_prev_process = mzapi_open_process();
 if(s_prev_process != ((struct ts_mzapi_process *)0))sleep(1);
 s_process = mzapi_open_process();
 if(s_process == ((struct ts_mzapi_process *)0))
 {
  if(s_prev_process != ((struct ts_mzapi_process *)0))(void)mzapi_close_process(s_prev_process);
  return(1);
 }
 mzps_title();
 s_pid = getpid();
 s_index = 0;
 s_trace = s_process;
 while(s_trace != ((struct ts_mzapi_process *)0))
 {
  s_index++;
  (void)fprintf(stdout,
   "%s %3d %5lu "
   "\x1b[1;32m" "%5lu" "\x1b[0m"
   " %-8s  %s  %7lu "
   "\x1b[1;36m" "%6lu" "\x1b[0m" "Kb "
   "\x1b[1;37m" "%6lu" "\x1b[0m" "Kb ",
   ((s_trace->ppid == s_pid) || (s_trace->pid == s_pid)) ? "*" : " ", 
   (int)s_index,
   (unsigned long)s_trace->ppid,
   (unsigned long)s_trace->pid,
   s_trace->username,
   (char *)(&s_trace->status[0]),
   (unsigned long)s_trace->threads,
   (unsigned long)s_trace->vmsize,
   (unsigned long)s_trace->rss);
  s_cpu_percent = (-1);
  s_prev_trace = s_prev_process;
  while(s_prev_trace != ((struct ts_mzapi_process *)0))
  {
   if(s_trace->pid == s_prev_trace->pid)
   {
    if((s_prev_trace->command_line != ((char *)0)) && (s_trace->command_line != ((char *)0)))
    {
     if(strcmp(s_trace->command_line, s_prev_trace->command_line) == 0)
     {
      if((s_trace->utime + s_trace->stime) >= (s_prev_trace->utime + s_prev_trace->stime))break;
     }
    }
    else if((s_prev_trace->process_name != ((char *)0)) && (s_trace->process_name != ((char *)0)))
    {
     if(strcmp(s_trace->process_name, s_prev_trace->process_name) == 0)
     {
      if((s_trace->utime + s_trace->stime) >= (s_prev_trace->utime + s_prev_trace->stime))break;
     }
    }
   }
   s_prev_trace = s_prev_trace->next;
  }
  if(s_prev_trace != ((struct ts_mzapi_process *)0))
  {
   time_t s_atom_utime, s_diff_utime;
#if defined(HZ)
   s_atom_utime = (time_t)HZ;
#else
   s_atom_utime = (time_t)100;
#endif
   s_diff_utime = (s_trace->utime + s_trace->stime) - (s_prev_trace->utime + s_prev_trace->stime);
   if(s_diff_utime > s_atom_utime)s_atom_utime = s_diff_utime;
   s_cpu_percent = (int)((s_diff_utime * ((time_t)1000)) / s_atom_utime);
  }
  if(s_cpu_percent >= 0)(void)fprintf(stdout, "%3d.%01d%% ", s_cpu_percent / 10, s_cpu_percent % 10);
  else (void)fprintf(stdout, "###.#%% ");
  if(s_trace->command_line != ((char *)0))
  {
#if 0 /* short process name */
   (void)fprintf(stdout,
    "\x1b[1;33m" "%-16s" "\x1b[0m",
    (s_trace->process_name != ((char *)0)) ? s_trace->process_name : s_trace->command_line);
#else /* full command line */
   (void)fprintf(stdout,
    "\x1b[1;33m" "%-16s" "\x1b[0m",
    (s_trace->command_line != ((char *)0)) ? s_trace->command_line : s_trace->process_name);
#endif
  }
  else
  {
   unsigned char s_temp[ 32 << 10 ];
   (void)sprintf((char *)(&s_temp[0]), "[" "\x1b[1;34m" "%s" "\x1b[0m" "]",
    (s_trace->process_name != ((char *)0)) ? s_trace->process_name : "<NULL>");
   (void)fprintf(stdout, "%-17s", (char *)(&s_temp[0]));
  }
  (void)fputs("\n", stdout);
#if 0 /* arg list - arg[argc] */
  do
  {
   int s_count;
   s_count = 0;
   while(s_count < s_trace->argc)
   {
    (void)fprintf(stdout,
     "\t"
     "\x1b[1;37m" "[" "\x1b[0m" "%2d" "\x1b[1;37m" "]" "\x1b[0m"
     " \"" "\x1b[1;37m" "%s" "\x1b[0m" "\"\n",
     (int)s_count,
     (s_trace->argv[s_count] != ((char *)0)) ? s_trace->argv[s_count] : "<NULL>");
    s_count++;
   }
  }while(0);
#endif
  (void)fflush(stdout);
  s_trace = s_trace->next;
 }
 mzps_title();
 (void)mzapi_close_process(s_process);
 if(s_prev_process != ((struct ts_mzapi_process *)0))(void)mzapi_close_process(s_prev_process);
 return(0);
}

#endif

/* vim: set expandtab: */
/* End of source */
