/*
 Copyright (C) JAEHYUK CHO
 All rights reserved.
 Code by JaeHyuk Cho <mailto:minzkn@minzkn.com>
*/

#if !defined(__def_mzapi_source_mzps_c__)
#define __def_mzapi_source_mzps_c__ "mzps.c"

#include "mzps.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <dirent.h>
#include <ctype.h>
#include <pwd.h>

#define __mzapi_peek_f__(m_cast,m_base,m_offset) ((m_cast)(((unsigned char *)(m_base)) + (m_offset)))

static int (__mzapi_peek_byte__)(const void *s_from, size_t s_offset);
static char * (__mzapi_poke_byte__)(void *s_to, size_t s_offset, int s_value);
static char * (__mzapi_skip_space__)(const char *s_string);
static char * (__mzapi_get_word_sep__)(const char *s_sep, char **s_pstring);
static char * (__mzapi_alloc_sprintf__)(const char *s_format, ...);
static struct ts_mzapi_process * (__mzapi_new_process__)(void);
static struct ts_mzapi_process * (__mzapi_free_process__)(struct ts_mzapi_process *s_process);
static int (__mzapi_command_line_decoder__)(const void * s_data, size_t s_size, int *s_argc_ptr, char ***s_argv_ptr);

struct ts_mzapi_process * (mzapi_open_process)(void);
struct ts_mzapi_process * (mzapi_close_process)(struct ts_mzapi_process *s_process);

static int (__mzapi_peek_byte__)(const void *s_from, size_t s_offset)
{
 return((int)(*__mzapi_peek_f__(unsigned char *, s_from, s_offset)));
}

static char * (__mzapi_poke_byte__)(void *s_to, size_t s_offset, int s_value)
{
 s_to = __mzapi_peek_f__(void *, s_to, s_offset);
 *((unsigned char *)s_to) = (unsigned char)s_value;
 return(s_to);
}

static char * (__mzapi_skip_space__)(const char *s_string)
{
 char *s_result;
 int s_byte;
 size_t s_offset;
 if(s_string == ((char *)0))return((char *)0);
 s_offset = (size_t)0;
 do
 {
  s_byte = __mzapi_peek_byte__(s_string, s_offset);
  if(isspace(s_byte) == 0)break;
  s_offset++;
 }while(1);
 s_result = __mzapi_peek_f__(char *, s_string, s_offset);
 return(s_result);
}

static char * (__mzapi_get_word_sep__)(const char *s_sep, char **s_pstring)
{
 char *s_result;
 char *s_string;
 int s_byte;
 int s_local_byte;
 size_t s_offset;
 size_t s_left;
 size_t s_right;
 size_t s_local_offset;
 if(s_pstring == ((char **)0))return((char *)0);
 s_string = *s_pstring;
 if(s_string == ((char *)0))return((char *)0);
 s_string = __mzapi_skip_space__(s_string);
 s_offset = (size_t)0;
 s_left = s_offset;
 s_right = s_offset;
 do
 {
  s_byte = __mzapi_peek_byte__(s_string, s_offset);
  if(s_byte == '\0')break;
  if(s_sep != ((char *)0))
  {
   s_local_offset = (size_t)0;
   do
   {
    s_local_byte = __mzapi_peek_byte__(s_sep, s_local_offset);
    if((s_byte == s_local_byte) || (s_local_byte == '\0'))break;
    s_local_offset++;
   }while(1);
   if(s_byte == s_local_byte)break;
  }
  else if(isspace(s_byte) != 0)break;
  s_offset++;
  if(isspace(s_byte) == 0)s_right = s_offset;
 }while(1);
 if(s_byte != '\0')s_offset++;
 if(s_right > s_left)
 {
  s_result = malloc((s_right - s_left) + ((size_t)1));
  if(s_result != ((char *)0))
  {
   (void)memcpy((void *)s_result, __mzapi_peek_f__(void *, s_string, s_left), s_right - s_left);
   (void)__mzapi_poke_byte__((void *)s_result, s_right - s_left, '\0');
  }
 }
 else s_result = (char *)0;
 *s_pstring = __mzapi_peek_f__(char *, s_string, s_offset);
 return(s_result);
}

static char * (__mzapi_alloc_sprintf__)(const char *s_format, ...)
{
 char *s_string;
 char *s_temp;
 size_t s_size;
 va_list s_var;
 if(s_format == ((char *)0))return((char *)0);
 va_start(s_var, s_format);
 s_size = (size_t)(4 << 10);
 do
 {
  s_string = (char *)malloc(s_size);
  if(s_string == ((char *)0))break;
  if(vsnprintf(s_string, s_size - ((size_t)1), s_format, s_var) < ((int)s_size))
  {
   s_temp = s_string;
   s_string = strdup(s_temp);
   free((void *)s_temp);
   break;
  }
  free((void *)s_string);
  s_string = (char *)0;
  s_size += (size_t)(4 << 10);
 }while(s_size < (512 << 10));
 va_end(s_var);
 return(s_string);
}

static struct ts_mzapi_process * (__mzapi_new_process__)(void)
{
 struct ts_mzapi_process *s_process;
 s_process = (struct ts_mzapi_process *)malloc((size_t)sizeof(struct ts_mzapi_process));
 if(s_process == ((struct ts_mzapi_process *)0))return((struct ts_mzapi_process *)0);
 s_process->next = (struct ts_mzapi_process *)0;
 s_process->pid = (pid_t)0;
 s_process->__command_line__ = (void *)0;
 s_process->__command_line_size__ = (size_t)0;
 s_process->argc = (int)0;
 s_process->argv = (char **)0;
 s_process->command_line = (char *)0;
 s_process->process_name = (char *)0;
 s_process->status[0] = (unsigned char)' ';
 s_process->status[1] = (unsigned char)' ';
 s_process->status[2] = (unsigned char)' ';
 s_process->status[3] = (unsigned char)'\0';
 s_process->ppid = (pid_t)0;
 s_process->uid = (uid_t)0;
 s_process->gid = (gid_t)0;
 s_process->username = (char *)0;
 s_process->vmsize = (size_t)0;
 s_process->threads = (int)0;
 s_process->utime = (time_t)0;
 s_process->stime = (time_t)0;
 s_process->nice = (int)0;
 s_process->rss = (size_t)0;
 return(s_process);
}

static struct ts_mzapi_process * (__mzapi_free_process__)(struct ts_mzapi_process *s_process)
{
 struct ts_mzapi_process *s_prev;
 while(s_process != ((struct ts_mzapi_process *)0))
 {
  s_prev = s_process;
  s_process = s_process->next;
  if(s_prev->username != ((char *)0))free((void *)s_prev->username);
  if(s_prev->process_name != ((char *)0))free((void *)s_prev->process_name);
  if(s_prev->command_line != ((char *)0))free((void *)s_prev->command_line);
  if(s_prev->argv != ((char **)0))free((void *)s_prev->argv);
  if(s_prev->__command_line__ != ((void *)0))free(s_prev->__command_line__);
  free((void *)s_prev);
 }
 return((struct ts_mzapi_process *)0);
}

static int (__mzapi_command_line_decoder__)(const void * s_data, size_t s_size, int *s_argc_ptr, char ***s_argv_ptr)
{
 int s_result;
 int s_argc;
 int s_byte;
 char **s_argv;
 size_t s_entry;
 size_t s_offset;
 char *s_arg;
 unsigned char s_fifo[32 << 10];
 size_t s_fifo_offset;
 if(s_argc_ptr != ((int *)0))*(s_argc_ptr) = (int)0;
 if(s_argv_ptr != ((char ***)0))*(s_argv_ptr) = (char **)0;
 s_argc = (int)0;
 s_argv = (char **)0;
 if((s_data == ((void *)0)) || (s_size <= ((size_t)0)))return(-1);
 s_result = 0;
 s_entry = (size_t)0;
 s_offset = (size_t)0;
 s_fifo_offset = (size_t)0;
 while(s_offset < s_size)
 {
  s_byte = __mzapi_peek_byte__(s_data, s_offset);
  if((s_byte == '\0') || ((s_offset + ((size_t)1)) >= s_size))
  {
   s_arg = __mzapi_peek_f__(char *, s_data, s_entry);
   *((void **)(&s_fifo[s_fifo_offset])) = (void *)s_arg;
   s_fifo_offset += (size_t)sizeof(void *);
   s_argc++;
   s_entry = s_offset + ((size_t)1);
  }
  s_offset++;         
 }
 if(s_fifo_offset > ((size_t)0))
 {
  s_argv = (char **)malloc(s_fifo_offset);
  if(s_argv != ((char **)0))
  {
   (void)memcpy((void *)s_argv, (void *)(&s_fifo[0]), s_fifo_offset);
   if(s_argc_ptr != ((int *)0))*(s_argc_ptr) = s_argc;
   if(s_argv_ptr != ((char ***)0))*(s_argv_ptr) = s_argv;
   s_result = 1;
  }
 }
 return(s_result);
}

struct ts_mzapi_process * (mzapi_open_process)(void)
{
 struct ts_mzapi_process *s_process;
 s_process = (struct ts_mzapi_process *)0;
 do
 {
  static const unsigned char c_path_proc[] = {"/proc"};
  DIR *s_directory;
  struct dirent *s_directory_entry;
  int s_temp;
  int s_check;
  char *s_proc_entry;
  struct stat s_status;
  char *s_this_entry;
  struct ts_mzapi_process *s_new;
  struct ts_mzapi_process *s_trace;
  char *s_line;
  char *s_sep_string;
  char *s_name;
  char *s_value;
  FILE *s_file;
  int s_handle;
  ssize_t s_read_bytes;
  unsigned char s_buffer[ 32 << 10 ];
  s_trace = s_process;
  s_directory = opendir((char *)c_path_proc);
  if(s_directory != ((DIR *)0))
  {
   do
   {
    s_directory_entry = readdir(s_directory);
    if(s_directory_entry == ((struct dirent *)0))break;
    if(isdigit(__mzapi_peek_byte__((void *)(s_directory_entry->d_name), (size_t)0)) != 0)
    {
     if(sscanf(s_directory_entry->d_name, "%i", &s_temp) == 1)
     {
      s_proc_entry = __mzapi_alloc_sprintf__("%s/%s", (char *)c_path_proc, (char *)s_directory_entry->d_name);
      if(s_proc_entry != ((char *)0))
      {
       if(stat((char *)s_proc_entry, (struct stat *)(&s_status)) == 0)
       {
        if(S_ISDIR(s_status.st_mode))
        {
         s_new = __mzapi_new_process__();
         if(s_new != ((struct ts_mzapi_process *)0))
         {
          s_new->pid = (pid_t)s_temp;
          /* cmdline */
          s_this_entry = __mzapi_alloc_sprintf__("%s/%s", (char *)s_proc_entry, "cmdline");
          if(s_this_entry != ((char *)0))
          {
	   s_handle = open(s_this_entry, O_RDONLY);
           if(s_handle != (-1))
           {
	    s_read_bytes = read(s_handle, (void *)(&s_buffer[0]), (size_t)(sizeof(s_buffer) - 1));
            if(s_read_bytes > ((ssize_t)0))
            {
             s_new->__command_line__ = malloc(s_read_bytes + ((size_t)1));
             if(s_new->__command_line__ != ((void *)0))
             {
	      (void)memcpy(s_new->__command_line__, (void *)(&s_buffer[0]), (size_t)s_read_bytes);
              (void)__mzapi_poke_byte__(s_new->__command_line__, (size_t)s_read_bytes, (int)'\0');
              s_new->__command_line_size__ = (size_t)s_read_bytes;
              (void)__mzapi_command_line_decoder__(s_new->__command_line__, s_new->__command_line_size__, (int *)(&s_new->argc), (char * **)(&s_new->argv));
              s_new->command_line = (char *)malloc(s_read_bytes + ((size_t)1));
              if(s_new->command_line != ((char *)0))
              {
               int s_byte;
               size_t s_offset;
               (void)memcpy((void *)s_new->command_line, s_new->__command_line__, (size_t)s_read_bytes);
               s_offset = (size_t)0;
               while(s_offset < ((size_t)s_read_bytes))
               {
                s_byte = __mzapi_peek_byte__((void *)s_new->command_line, s_offset);
                if((s_byte == '\0') && ((s_offset + ((size_t)1)) < ((size_t)s_read_bytes)))
                {
                 (void)__mzapi_poke_byte__((void *)s_new->command_line, s_offset, (int)' ');
                }
                s_offset++;
               }
               (void)__mzapi_poke_byte__((void *)s_new->command_line, s_offset, (int)'\0');
              }
             }
            } 
            (void)close(s_handle);
           }
           free((void *)s_this_entry); 
          }
          /* status */
          s_this_entry = __mzapi_alloc_sprintf__("%s/%s", (char *)s_proc_entry, "status");
          if(s_this_entry != ((char *)0))
          {
           s_file = fopen(s_this_entry, "rb");
           if(s_file != ((FILE *)0))
           {
            do
            {
             s_line = fgets((char *)(&s_buffer[0]), sizeof(s_buffer) - 1, s_file);
             if(s_line == ((char *)0))break;
             s_sep_string = s_line;
             s_name = __mzapi_get_word_sep__(":", (char * *)(&s_sep_string));
             s_value = __mzapi_get_word_sep__("\n", (char * *)(&s_sep_string));
             if((s_name != ((char *)0)) && (s_value != ((char *)0)))
             {
              if(strcasecmp(s_name, "name") == 0)
              {
               if(s_new->process_name != ((char *)0))free((void *)s_new->process_name);
               s_new->process_name = s_value;
               s_value = (char *)0;
              }
              else if(strcasecmp(s_name, "state") == 0)
              {
               s_new->status[0] = *((unsigned char *)s_value);
              }
              else if(strcasecmp(s_name, "pid") == 0)s_new->pid = (pid_t)atoi(s_value);
              else if(strcasecmp(s_name, "ppid") == 0)s_new->ppid = (pid_t)atoi(s_value);
              else if(strcasecmp(s_name, "uid") == 0)s_new->uid = (uid_t)atoi(s_value);
              else if(strcasecmp(s_name, "gid") == 0)s_new->gid = (gid_t)atoi(s_value);
              else if(strcasecmp(s_name, "vmsize") == 0)s_new->vmsize = (size_t)atoi(s_value);
              else if(strcasecmp(s_name, "threads") == 0)s_new->threads = (int)atoi(s_value);
             }
             if(s_name != ((char *)0))free((void *)s_name);
             if(s_value != ((char *)0))free((void *)s_value);
            }while(1); 
            (void)fclose(s_file);
           }
           free((void *)s_this_entry); 
          }
          /* stat */
          s_this_entry = __mzapi_alloc_sprintf__("%s/%s", (char *)s_proc_entry, "stat");
          if(s_this_entry != ((char *)0))
          {
           s_handle = open(s_this_entry, O_RDONLY);
           if(s_handle != (-1))
           {
	    s_read_bytes = read(s_handle, (void *)(&s_buffer[0]), (size_t)(sizeof(s_buffer) - 1));
            if(s_read_bytes > ((ssize_t)0))
            {
	      s_buffer[s_read_bytes] = (unsigned char)'\0';
              do
              {
               char * s_sep_string2;
               s_sep_string = strstr((char *)(&s_buffer[0]), ")");
               s_sep_string2 = s_sep_string;
               while(s_sep_string2 != ((char *)0))
               {
                s_sep_string2 = strstr(__mzapi_peek_f__(char *, s_sep_string2, 1), ")");
                if(s_sep_string2 != ((char *)0))s_sep_string = s_sep_string2;
               }
               if(s_sep_string != ((char *)0))
               {
                unsigned long s_utime;
                unsigned long s_stime;
                long s_nice;
                unsigned long s_rss;
                s_sep_string = __mzapi_peek_f__(char *, s_sep_string, 1);
                s_check = sscanf((char *)s_sep_string,
                 /* "* (*)" */ /* pid (process_name) */
                 "%*s " /* state */
                 "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s "
                 "%lu %lu " /* utime stime */
                 "%*s %*s %*s "
                 "%ld " /* nice */
                 "%*s %*s %*s %*s "
                 "%lu" /* rss */,
                 &s_utime, &s_stime,
                 &s_nice, &s_rss    
                ); 
                if(s_check == 4)
                {
                 s_new->utime = (time_t)s_utime;
                 s_new->stime = (time_t)s_stime;
                 s_new->nice = (int)s_nice;
                 s_new->rss = ((size_t)s_rss) * (getpagesize() >> 10);
                }
               }
              }while(0);
            } 
            (void)close(s_handle);
           }
           free((void *)s_this_entry); 
          }
          /* check */
          if((s_new->rss <= ((size_t)0)) && (s_new->status[0] != ((unsigned char)'Z')))s_new->status[1] = (unsigned char)'W';
          if(s_new->nice < 0)s_new->status[2] = (unsigned char)'<';
          else if(s_new->nice > 0)s_new->status[2] = (unsigned char)'N'; 
	  /* username */
          do
	  {
           struct passwd *s_password_entry;
           s_password_entry = getpwuid(s_new->uid);
           if(s_password_entry != ((struct passwd *)0))s_new->username = strdup(s_password_entry->pw_name);
           else s_new->username = strdup("owner");
	  }while(0);
          /* append */
          if(s_trace != ((struct ts_mzapi_process *)0))s_trace->next = s_new;
          else s_process = s_new;
          s_trace = s_new;        
         }
        }
       }
       free((void *)s_proc_entry);
      }
     } 
    }
   }while(1);
   (void)closedir(s_directory);
  }
 }while(0);
 return(s_process);
}

struct ts_mzapi_process * (mzapi_close_process)(struct ts_mzapi_process *s_process)
{
 return(__mzapi_free_process__(s_process));
}

#endif

/* vim: set expandtab: */
/* End of source */
